#-----------------------------------------------------------------------------
# Add mags for each conforming part
# Copyright (c) 2003 by David G. Drumright (ockham)
# April 13, 2003
#
# Custom for Compiler
#-----------------------------------------------------------------------------
# Select a figure.
# Activate the script.
# For all magnets on the figure, we spawn a new magnet
# with the same parameters, on the corresponding part of
# each clothing item conformed to the figure.

#-----------------------------------------------------------------------------

import string
import poser
scene = poser.Scene()
scene.SetFrame(0)

#-----------------------------------------------------------------------------

def CreateMagsOutward(MainFigArg, MainActorArg, MainBaseArg, MainZoneArg, MainMagArg):
    # First find out if there are any conforming figures.
    for OneFig in scene.Figures():
        if OneFig.Name() == MainFigArg.Name(): # Don't act on the base!
            continue
        elif OneFig.ConformTarget().InternalName() == MainFigArg.InternalName():
            # This figure is conformed to the main figure.
            # Make a magnet on the new part.
            # The transfer has to be done by name.
            try:
                NewPart = OneFig.ActorByInternalName(MainActorArg.InternalName())
            except: # Clothing doesn't have a corresponding part: this is OK.
                return # Just skip the magnet
            # Now make a magnet on this part.
            try:
                scene.SelectActor(NewPart)
                scene.CreateMagnet()
                NewMag = scene.CurrentActor()
                # Because CreateMagnet doesn't return the object but
                # leaves the new mag selected, we have to get the object
                # by picking up the selection immediately.
            except:
                raise "Can't make new magnet"

            # Put all the parms of the original mag onto the new.
            for OneParm in MainMagArg.Parameters():
                ParmName = OneParm.Name()
                Val = OneParm.Value()
                NewParm = NewMag.Parameter(ParmName)
                NewParm.SetValue(Val)

            # Locate the new base by the number-finding method
            NameParts = string.split(NewMag.Name())
            Num = int(NameParts[1])
            BaseNameShouldBe = "Mag Base %d" % Num
            for TryBase in scene.Actors():
                if TryBase.Name() == BaseNameShouldBe:
                   NewBase = TryBase
            # Put all the parms of the original base onto the new:
            for OneParm in MainBaseArg.Parameters():
                ParmName = OneParm.Name()
                Val = OneParm.Value()
                NewParm = NewBase.Parameter(ParmName)
                NewParm.SetValue(Val)

            # Locate the new zone by the number-finding method
            NameParts = string.split(NewMag.Name())
            Num = int(NameParts[1])
            ZoneNameShouldBe = "Mag Zone %d" % Num
            for TryZone in scene.Actors():
                if TryZone.Name() == ZoneNameShouldBe:
                   NewZone = TryZone
            # Put all the parms of the original zone onto the new:
            for OneParm in MainZoneArg.Parameters():
                ParmName = OneParm.Name()
                Val = OneParm.Value()
                NewParm = NewZone.Parameter(ParmName)
                NewParm.SetValue(Val) # copy others

#-----------------------------------------------------------------------------

MagName = "Mag"
OneMag = 0
OneBase = 0
OneZone = 0

# See if the currently selected figure has any magnets.
OneFig = scene.CurrentFigure()

# The Base and Zone are both childs of the Actor.
# The Mag is a child of the Base.
# So we look first for Base and Zone, then get the Mag
# as the only child of Base.

for OneActor in OneFig.Actors():
    for TryPart in OneActor.Children():
        if TryPart.IsBase(): # this is a base
            OneBase = TryPart
            # The Magnet itself is the Base's only child:
            for TryChild in OneBase.Children():
                TryName = TryChild.InternalName()
                OneMag = TryChild
                # At this point we have a magnet.
                # We still need to find the corresponding zone.
                ColonIndex = string.find(TryName,":")
                if ColonIndex == (-1):
                    NameParts = string.split(TryName)
                else:
                    TryName = TryName[:ColonIndex]
                    NameParts = string.split(TryName)
                Num = int(NameParts[1])
                ZoneNameShouldBe = "Mag Zone %d" % Num
                for TryZone in scene.Actors():
                    ColonIndex = string.find(TryZone.InternalName(),":")
                    if ColonIndex == (-1):
                        TryName = TryZone.InternalName()
                    else:
                        TryName = TryZone.InternalName()[:ColonIndex]
                    if TryName == ZoneNameShouldBe:
                        OneZone = TryZone
                # Run the routine that sets up identical mags
                # on all conformed Figures, on the same body part.
                if OneMag and OneBase and OneZone:
                    CreateMagsOutward(OneFig,OneActor,OneBase,OneZone,OneMag)
                else:
                   raise "Can't create magnet for %s" % OneMag.Name()


# That's all, folks!
#-----------------------------------------------------------------------------
